@@ -139,8 +139,14 @@ class PanelExecutor(WSHandler):
     """
 
     _code = """
+    import os
+    import pathlib
+
+    app = '{{ path }}'
+    os.chdir(str(pathlib.Path(app).parent))
+
     from panel.io.jupyter_server_extension import PanelExecutor
-    executor = PanelExecutor('{{ path }}', '{{ token }}', '{{ root_url }}')
+    executor = PanelExecutor(app, '{{ token }}', '{{ root_url }}')
     executor.render()
     """
 
@@ -156,6 +162,7 @@ class PanelExecutor(WSHandler):
         self.receiver = Receiver(self.protocol)
         self.handler = ProtocolHandler()
         self.write_lock = tornado.locks.Lock()
+        self._context = None
 
         self.resources = Resources(
             mode="server", root_url=self.root_url,
@@ -164,7 +171,6 @@ class PanelExecutor(WSHandler):
         self._set_state()
         self.session = self._create_server_session()
         self.connection = ServerConnection(self.protocol, self, None, self.session)
-        publish_display_data({'application/bokeh-extensions': extension_dirs})
 
     def _get_payload(self, token: str) -> Dict[str, Any]:
         payload = get_token_payload(token)
@@ -177,6 +183,7 @@ class PanelExecutor(WSHandler):
         return payload
 
     def _set_state(self):
+        state._jupyter_kernel_context = True
         with edit_readonly(state):
             state.base_url = self.root_url + '/'
             state.rel_path = self.root_url
@@ -210,7 +217,7 @@ class PanelExecutor(WSHandler):
     def _create_server_session(self) -> ServerSession:
         doc = Document()
 
-        session_context = BokehSessionContext(
+        self._context = session_context = BokehSessionContext(
             self.session_id, None, doc
         )
 
@@ -226,7 +233,13 @@ class PanelExecutor(WSHandler):
         # use the _attribute to set the public property .session_context
         doc._session_context = weakref.ref(session_context)
 
-        app = build_single_handler_application(self.path)
+        if self.path.endswith('.yaml') or self.path.endswith('.yml'):
+            from lumen.command import (
+                build_single_handler_application as build_lumen_app,
+            )
+            app = build_lumen_app(self.path, argv=None)
+        else:
+            app = build_single_handler_application(self.path)
         with set_curdoc(doc):
             app.initialize_document(doc)
 
@@ -266,7 +279,7 @@ class PanelExecutor(WSHandler):
     ) -> None:
         metadata = {'binary': binary}
         if binary:
-            self.comm.send({}, metadata=metadata, buffers=[binary])
+            self.comm.send({}, metadata=metadata, buffers=[message])
         else:
             self.comm.send(message, metadata=metadata)
 
@@ -275,13 +288,16 @@ class PanelExecutor(WSHandler):
         Renders the application to an IPython.display.HTML object to
         be served by the `PanelJupyterHandler`.
         """
-        return HTML(server_html_page_for_session(
-            self.session,
-            resources=self.resources,
-            title=self.session.document.title,
-            template=self.session.document.template,
-            template_variables=self.session.document.template_variables
-        ))
+        with set_curdoc(self.session.document):
+            html = server_html_page_for_session(
+                self.session,
+                resources=self.resources,
+                title=self.session.document.title,
+                template=self.session.document.template,
+                template_variables=self.session.document.template_variables
+            )
+        publish_display_data({'application/bokeh-extensions': extension_dirs})
+        return HTML(html)
 
 
 class PanelJupyterHandler(JupyterHandler):
@@ -534,10 +550,12 @@ class PanelWSProxy(WSHandler, JupyterHandler):
                 return self._internal_error(content['data'])
             binary = metadata.get('binary')
             if binary:
-                data = msg['buffers'][0]
+                fragment = msg['buffers'][0].tobytes()
             else:
-                data = content['data']
-            message = await self._receive(data)
+                fragment = content['data']
+                if isinstance(fragment, dict):
+                    fragment = json.dumps(fragment)
+            message = await self._receive(fragment)
             if message:
                 await self.send_message(message)
 
