@@ -31,23 +31,29 @@ other layouts, then Panel will automatically wrap it in the right pane.
 For more detail see the Getting Started Guide
 https://panel.holoviz.org/getting_started/index.html
 """
-from .alert import Alert # noqa
-from .base import PaneBase, Pane, panel # noqa
-from .equation import LaTeX # noqa
-from .deckgl import DeckGL # noqa
-from .echarts import ECharts # noqa
-from .holoviews import HoloViews, Interactive # noqa
-from .idom import IDOM # noqa0
-from .ipywidget import IPyWidget # noqa
-from .image import GIF, ICO, JPG, PDF, PNG, SVG # noqa
-from .markup import DataFrame, HTML, JSON, Markdown, Str # noqa
-from .media import Audio, Video # noqa
-from .perspective import Perspective # noqa
-from .plotly import Plotly # noqa
-from .plot import Bokeh, Matplotlib, RGGPlot, YT # noqa
-from .streamz import Streamz # noqa
-from .vega import Vega # noqa
-from .vtk import VTKVolume, VTK # noqa
+from .alert import Alert  # noqa
+from .base import Pane, PaneBase, panel  # noqa
+from .deckgl import DeckGL  # noqa
+from .echarts import ECharts  # noqa
+from .equation import LaTeX  # noqa
+from .holoviews import HoloViews, Interactive  # noqa
+from .idom import IDOM  # noqa
+from .image import (  # noqa
+    GIF, ICO, JPG, PDF, PNG, SVG,
+)
+from .ipywidget import IPyLeaflet, IPyWidget, Reacton  # noqa
+from .markup import (  # noqa
+    HTML, JSON, DataFrame, Markdown, Str,
+)
+from .media import Audio, Video  # noqa
+from .perspective import Perspective  # noqa
+from .plot import (  # noqa
+    YT, Bokeh, Matplotlib, RGGPlot,
+)
+from .plotly import Plotly  # noqa
+from .streamz import Streamz  # noqa
+from .vega import Vega  # noqa
+from .vtk import VTK, VTKVolume  # noqa
 
 __all__ = (
     "Alert",
@@ -63,6 +69,7 @@ __all__ = (
     "IDOM",
     "Interactive",
     "IPyWidget",
+    "IPyLeaflet",
     "JPG",
     "JSON",
     "LaTeX",
@@ -75,6 +82,7 @@ __all__ = (
     "Perspective",
     "Plotly",
     "PNG",
+    "Reacton",
     "RGGPlot",
     "Str",
     "Streamz",
