@@ -1,17 +1,26 @@
+from __future__ import annotations
+
 import sys
 
-from functools import partial
+from typing import (
+    TYPE_CHECKING, Any, ClassVar, Mapping, Optional,
+)
 
-import param
 import numpy as np
+import param
 
 from bokeh.models import ColumnDataSource
 from pyviz_comms import JupyterComm
 
-from ..viewable import Layoutable
 from ..util import lazy_load
+from ..viewable import Layoutable
 from .base import PaneBase
 
+if TYPE_CHECKING:
+    from bokeh.document import Document
+    from bokeh.model import Model
+    from pyviz_comms import Comm
+
 
 def ds_as_cds(dataset):
     """
@@ -103,11 +112,11 @@ class Vega(PaneBase):
         'excel', 'ggplot2', 'quartz', 'vox', 'fivethirtyeight', 'dark',
         'latimes', 'urbaninstitute', 'googlecharts'])
 
-    priority = 0.8
+    priority: ClassVar[float | bool | None] = 0.8
 
-    _rename = {'selection': None, 'debounce': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {'selection': None, 'debounce': None}
 
-    _updates = True
+    _updates: ClassVar[bool] = True
 
     def __init__(self, object=None, **params):
         super().__init__(object, **params)
@@ -145,7 +154,7 @@ class Vega(PaneBase):
         return False
 
     @classmethod
-    def applies(cls, obj):
+    def applies(cls, obj: Any) -> float | bool | None:
         if isinstance(obj, dict) and 'vega' in obj.get('$schema', '').lower():
             return True
         return cls.is_altair(obj)
@@ -236,7 +245,10 @@ class Vega(PaneBase):
             value = list(value)
         self.selection.param.update(**{name: value})
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
+    def _get_model(
+        self, doc: Document, root: Optional[Model] = None,
+        parent: Optional[Model] = None, comm: Optional[Comm] = None
+    ) -> Model:
         VegaPlot = lazy_load('panel.models.vega', 'VegaPlot', isinstance(comm, JupyterComm), root)
         sources = {}
         if self.object is None:
@@ -250,16 +262,13 @@ class Vega(PaneBase):
             data=json, data_sources=sources, events=list(self._selections),
             throttle=self._throttle, **props
         )
-        if comm:
-            model.on_event('vega_event', self._comm_event)
-        else:
-            model.on_event('vega_event', partial(self._server_event, doc))
+        self._register_events('vega_event', model=model, doc=doc, comm=comm)
         if root is None:
             root = model
         self._models[root.ref['id']] = (model, parent)
         return model
 
-    def _update(self, ref=None, model=None):
+    def _update(self, ref: str, model: Model) -> None:
         if self.object is None:
             json = None
         else:
