@@ -5,6 +5,8 @@ import pathlib
 
 import param
 
+from ...config import config
+from ...io.resources import JS_URLS
 from ...layout import Card
 from ..base import BasicTemplate
 from ..theme import DarkTheme, DefaultTheme
@@ -14,6 +16,9 @@ class GoldenTemplate(BasicTemplate):
     """
     GoldenTemplate is built on top of golden-layout library.
     """
+    sidebar_width = param.Integer(20, doc="""
+        The width of the sidebar in percent. Default is 20.""")
+
     _css = pathlib.Path(__file__).parent / 'golden.css'
 
     _template = pathlib.Path(__file__).parent / 'golden.html'
@@ -27,11 +32,11 @@ class GoldenTemplate(BasicTemplate):
 
     _resources = {
         'css': {
-            'goldenlayout': "https://golden-layout.com/files/latest/css/goldenlayout-base.css",
+            'goldenlayout': f"{config.npm_cdn}/golden-layout@1.5.9/src/css/goldenlayout-base.css",
         },
         'js': {
-            'jquery': "http://code.jquery.com/jquery-1.11.1.min.js",
-            'goldenlayout': "https://golden-layout.com/files/latest/js/goldenlayout.js"
+            'jquery': JS_URLS['jQuery'],
+            'goldenlayout': f"{config.npm_cdn}/golden-layout@1.5.9/dist/goldenlayout.min.js"
         }
     }
 
