@@ -6,15 +6,17 @@ The doc_available tests check that python in markdown files can be run top to bo
 
 """
 import ast
+import runpy
 
 from inspect import isclass
 from pathlib import Path
-from subprocess import run
 
 import pytest
 
 import panel as pn
 
+pytestmark = pytest.mark.docs
+
 REF_PATH = Path(__file__).parents[2] / "examples" / "reference"
 ref_available = pytest.mark.skipif(not REF_PATH.is_dir(), reason="folder 'examples/reference' not found")
 
@@ -55,7 +57,7 @@ def test_widgets_are_in_reference_gallery():
 
 @ref_available
 def test_panes_are_in_reference_gallery():
-    exceptions = {"PaneBase", "YT", "RGGPlot", "Interactive", "ICO"}
+    exceptions = {"PaneBase", "YT", "RGGPlot", "Interactive", "ICO", "IPyLeaflet"}
     docs = {f.with_suffix("").name for f in (REF_PATH / "panes").iterdir()}
 
     def is_panel_pane(attr):
@@ -70,7 +72,7 @@ def test_panes_are_in_reference_gallery():
 @pytest.mark.parametrize(
     "file", doc_files, ids=[str(f.relative_to(DOC_PATH)) for f in doc_files]
 )
-def test_markdown_codeblocks(file):
+def test_markdown_codeblocks(file, tmp_path):
     from markdown_it import MarkdownIt
 
     exceptions = ("await", "pn.serve", "django")
@@ -82,8 +84,17 @@ def test_markdown_codeblocks(file):
             if "pyodide" in n.info.lower() or "python" in n.info.lower():
                 if ">>>" not in n.content:
                     lines += n.content
-    if lines:
-        ast.parse(lines)
+    if not lines:
+        return
 
-    if lines and not any(w in lines for w in exceptions):
-        run(["python", "-c", lines], timeout=30, check=True)
+    ast.parse(lines)
+
+    if any(w in lines for w in exceptions):
+        return
+
+    mod = tmp_path / f'{file.stem}.py'
+
+    with open(mod, 'w', encoding='utf-8') as f:
+        f.writelines(lines)
+
+    runpy.run_path(mod)
