@@ -2,13 +2,19 @@
 The Debugger Widget is an uneditable Card that gives you feedback on errors
 thrown by your Panel callbacks.
 """
-import param
+from __future__ import annotations
+
 import logging
 
-from ..layout.card import Card
-from ..reactive import ReactiveHTML
+from typing import (
+    ClassVar, Dict, List, Mapping,
+)
+
+import param
+
 from ..io.state import state
-from ..layout import Row, HSpacer
+from ..layout import Card, HSpacer, Row
+from ..reactive import ReactiveHTML
 from .terminal import Terminal
 
 
@@ -35,7 +41,6 @@ class TermFormatter(logging.Formatter):
         self.only_last = only_last
 
     def format(self, record):
-
         record.message = record.getMessage()
         if self.usesTime():
             record.asctime = self.formatTime(record, self.datefmt)
@@ -114,7 +119,7 @@ class DebuggerButtons(ReactiveHTML):
 
     clears = param.Integer(default=0)
 
-    _template = """
+    _template: ClassVar[str] = """
     <div class="bk" style="display: flex;">
       <button class='special_btn clear_btn'
          id="clear_btn"
@@ -132,7 +137,7 @@ class DebuggerButtons(ReactiveHTML):
     </div>
     """
 
-    js_cb = """
+    js_cb: ClassVar[str] = """
         var filename = data.debug_name+'.txt'
         console.log('saving debugger terminal output to '+filename)
         var blob = new Blob([data.terminal_output],
@@ -153,12 +158,12 @@ class DebuggerButtons(ReactiveHTML):
         }
         """
 
-    _scripts = {
+    _scripts: ClassVar[Dict[str, str | List[str]]] = {
         'click': js_cb,
         'click_clear': "data.clears += 1"
     }
 
-    _dom_events = {'clear_btn': ['click']}
+    _dom_events: ClassVar[Dict[str, List[str]]] = {'clear_btn': ['click']}
 
 
 class Debugger(Card):
@@ -201,9 +206,8 @@ class Debugger(Card):
         Loggers which will be prompted in the debugger terminal."""
     )
 
-    _rename = Card._rename.copy()
-
-    _rename.update({
+    _rename: ClassVar[Mapping[str, str | None]] = dict(
+        Card._rename, **{
         '_number_of_errors': None,
         '_number_of_warnings': None,
         '_number_of_infos': None,
@@ -259,6 +263,7 @@ class Debugger(Card):
             logger.addHandler(stream_handler)
 
         self.terminal = terminal
+        self.stream_handler = stream_handler
 
         #callbacks for header
         self.param.watch(self.update_log_counts,'_number_of_errors')
@@ -309,7 +314,11 @@ class Debugger(Card):
 
         self.title = ', '.join(title)
 
-    def acknowledge_errors(self,event):
+    def acknowledge_errors(self, event):
         self._number_of_errors = 0
         self._number_of_warnings = 0
         self._number_of_infos = 0
+
+    @param.depends("level", watch=True)
+    def _update_level(self):
+        self.stream_handler.setLevel(self.level)
