@@ -2,11 +2,16 @@
 The input widgets generally allow entering arbitrary information into
 a text field or similar.
 """
+from __future__ import annotations
+
 import ast
 import json
 
 from base64 import b64decode
-from datetime import datetime, date
+from datetime import date, datetime
+from typing import (
+    TYPE_CHECKING, Any, ClassVar, Dict, Mapping, Optional, Type,
+)
 
 import numpy as np
 import param
@@ -14,17 +19,24 @@ import param
 from bokeh.models.formatters import TickFormatter
 from bokeh.models.widgets import (
     CheckboxGroup as _BkCheckboxGroup, ColorPicker as _BkColorPicker,
-    DatePicker as _BkDatePicker, Div as _BkDiv, TextInput as _BkTextInput,
-    PasswordInput as _BkPasswordInput, Spinner as _BkSpinner,
-    FileInput as _BkFileInput, TextAreaInput as _BkTextAreaInput,
-    NumericInput as _BkNumericInput
+    DatePicker as _BkDatePicker, Div as _BkDiv, FileInput as _BkFileInput,
+    NumericInput as _BkNumericInput, PasswordInput as _BkPasswordInput,
+    Spinner as _BkSpinner, TextAreaInput as _BkTextAreaInput,
+    TextInput as _BkTextInput,
 )
 
 from ..config import config
-from ..layout import Column
-from ..util import param_reprs
-from .base import Widget, CompositeWidget
+from ..layout import Column, Panel
 from ..models import DatetimePicker as _bkDatetimePicker
+from ..util import param_reprs
+from .base import CompositeWidget, Widget
+
+if TYPE_CHECKING:
+    from bokeh.document import Document
+    from bokeh.model import Model
+    from pyviz_comms import Comm
+
+    from ..viewable import Viewable
 
 
 class TextInput(Widget):
@@ -50,7 +62,29 @@ class TextInput(Widget):
     value_input = param.String(default='', allow_None=True, doc="""
       Initial or entered text value updated on every key press.""")
 
-    _widget_type = _BkTextInput
+    _widget_type: ClassVar[Type[Model]] = _BkTextInput
+
+    @classmethod
+    def from_param(cls, parameter: param.Parameter, onkeyup=False, **params) -> Viewable:
+        """
+        Construct a widget from a Parameter and link the two
+        bi-directionally.
+
+        Parameters
+        ----------
+        parameter: param.Parameter
+          A parameter to create the widget from.
+        onkeyup: boolean
+          Whether to trigger events on every key press.
+        params: dict
+          Keyword arguments to be passed to the widget constructor
+
+        Returns
+        -------
+        Widget instance linked to the supplied parameter
+        """
+        params['onkeyup'] = onkeyup
+        return super().from_param(parameter, **params)
 
 
 class PasswordInput(TextInput):
@@ -67,7 +101,7 @@ class PasswordInput(TextInput):
     ... )
     """
 
-    _widget_type = _BkPasswordInput
+    _widget_type: ClassVar[Type[Model]] = _BkPasswordInput
 
 
 class TextAreaInput(TextInput):
@@ -86,7 +120,7 @@ class TextAreaInput(TextInput):
     ... )
     """
 
-    _widget_type = _BkTextAreaInput
+    _widget_type: ClassVar[Type[Model]] = _BkTextAreaInput
 
 
 class FileInput(Widget):
@@ -119,11 +153,13 @@ class FileInput(Widget):
 
     value = param.Parameter(default=None)
 
-    _widget_type = _BkFileInput
+    _rename: ClassVar[Mapping[str, str | None]] = {'name': None, 'filename': None}
 
-    _source_transforms = {'value': "'data:' + source.mime_type + ';base64,' + value"}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': "'data:' + source.mime_type + ';base64,' + value"
+    }
 
-    _rename = {'name': None, 'filename': None}
+    _widget_type: ClassVar[Type[Model]] = _BkFileInput
 
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
@@ -168,9 +204,9 @@ class FileInput(Widget):
                 "filenames was given. Please provide a single filename "
                 "or file-like object."
             )
-        elif not isinstance(value, list):
+        if not isinstance(value, list):
             value = [self.value]
-        elif not isinstance(filename, list):
+        if not isinstance(filename, list):
             filename = [filename]
 
         for val, fn in zip(value, filename):
@@ -199,15 +235,19 @@ class StaticText(Widget):
     value = param.Parameter(default=None, doc="""
         The current value""")
 
-    _format = '<b>{title}</b>: {value}'
+    _format: ClassVar[str] = '<b>{title}</b>: {value}'
 
-    _rename = {'name': None, 'value': 'text'}
+    _rename: ClassVar[Mapping[str, str | None]] = {'name': None, 'value': 'text'}
 
-    _target_transforms = {'value': 'target.text.split(": ")[0]+": "+value'}
+    _target_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': 'target.text.split(": ")[0]+": "+value'
+    }
 
-    _source_transforms = {'value': 'value.split(": ")[1]'}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': 'value.split(": ")[1]'
+    }
 
-    _widget_type = _BkDiv
+    _widget_type: ClassVar[Type[Model]] = _BkDiv
 
     def _process_param_change(self, msg):
         msg = super()._process_property_change(msg)
@@ -249,17 +289,22 @@ class DatePicker(Widget):
 
     enabled_dates = param.List(default=None, class_=(date, str))
 
-    _source_transforms = {}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {}
 
-    _rename = {'start': 'min_date', 'end': 'max_date', 'name': 'title'}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'start': 'min_date', 'end': 'max_date', 'name': 'title'
+    }
 
-    _widget_type = _BkDatePicker
+    _widget_type: ClassVar[Type[Model]] = _BkDatePicker
 
     def _process_property_change(self, msg):
         msg = super()._process_property_change(msg)
-        if 'value' in msg:
-            if isinstance(msg['value'], str):
-                msg['value'] = datetime.date(datetime.strptime(msg['value'], '%Y-%m-%d'))
+        for p in ('start', 'end', 'value'):
+            if p not in msg:
+                continue
+            value = msg[p]
+            if isinstance(value, str):
+                msg[p] = datetime.date(datetime.strptime(value, '%Y-%m-%d'))
         return msg
 
 
@@ -278,23 +323,23 @@ class _DatetimePickerBase(Widget):
       Enable editing of the seconds in the widget.""")
 
     end = param.Date(default=None, doc="""
-      Inclusive upper bound of the allowed date selection. Note that while
-      the parameter accepts datetimes the time portion of the range
-      is ignored.""")
+      Inclusive upper bound of the allowed date selection.""")
 
     military_time = param.Boolean(default=True, doc="""
       Whether to display time in 24 hour format.""")
 
     start = param.Date(default=None, doc="""
-      Inclusive lower bound of the allowed date selection. Note that while
-      the parameter accepts datetimes the time portion of the range
-      is ignored.""")
+      Inclusive lower bound of the allowed date selection.""")
 
-    _source_transforms = {'value': None, 'start': None, 'end': None, 'mode': None}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': None, 'start': None, 'end': None, 'mode': None
+    }
 
-    _rename = {'start': 'min_date', 'end': 'max_date', 'name': 'title'}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'start': 'min_date', 'end': 'max_date', 'name': 'title'
+    }
 
-    _widget_type = _bkDatetimePicker
+    _widget_type: ClassVar[Type[Model]] = _bkDatetimePicker
 
     __abstract = True
 
@@ -303,15 +348,17 @@ class _DatetimePickerBase(Widget):
         self._update_value_bounds()
 
     @staticmethod
-    def _date_to_datetime(x):
-        if isinstance(x, date):
-            return datetime(x.year, x.month, x.day)
+    def _convert_to_datetime(v):
+        if isinstance(v, datetime):
+            return v
+        elif isinstance(v, date):
+            return datetime(v.year, v.month, v.day)
 
     @param.depends('start', 'end', watch=True)
     def _update_value_bounds(self):
         self.param.value.bounds = (
-            self._date_to_datetime(self.start),
-            self._date_to_datetime(self.end),
+            self._convert_to_datetime(self.start),
+            self._convert_to_datetime(self.end)
         )
         self.param.value._validate(self.value)
 
@@ -325,6 +372,10 @@ class _DatetimePickerBase(Widget):
         msg = super()._process_param_change(msg)
         if 'value' in msg:
             msg['value'] = self._deserialize_value(msg['value'])
+        if 'min_date' in msg:
+            msg['min_date'] = self._convert_to_datetime(msg['min_date'])
+        if 'max_date' in msg:
+            msg['max_date'] = self._convert_to_datetime(msg['max_date'])
         return msg
 
 
@@ -353,12 +404,6 @@ class DatetimePicker(_DatetimePickerBase):
         if isinstance(value, str) and value:
             value = datetime.strptime(value, r'%Y-%m-%d %H:%M:%S')
 
-            # Hour, minute and seconds can be increased after end is reached.
-            # This forces the hours, minute and second to be 0.
-            end = self._date_to_datetime(self.end)
-            if end is not None and value > end:
-                value = end
-
         return value
 
     def _deserialize_value(self, value):
@@ -396,13 +441,6 @@ class DatetimeRangePicker(_DatetimePickerBase):
                 for value in value.split(' to ')
             ]
 
-            # Hour, minute and seconds can be increased after end is reached.
-            # This forces the hours, minute and second to be 0.
-            end = self._date_to_datetime(self.end)
-            if end is not None and value[0] > end:
-                value[0] = end
-            if end is not None and value[1] > end:
-                value[1] = end
 
             value = tuple(value)
 
@@ -432,9 +470,9 @@ class ColorPicker(Widget):
     value = param.Color(default=None, doc="""
         The selected color""")
 
-    _widget_type = _BkColorPicker
+    _widget_type: ClassVar[Type[Model]] = _BkColorPicker
 
-    _rename = {'value': 'color', 'name': 'title'}
+    _rename: ClassVar[Mapping[str, str | None]] = {'value': 'color', 'name': 'title'}
 
 
 class _NumericInputBase(Widget):
@@ -454,9 +492,9 @@ class _NumericInputBase(Widget):
     end = param.Parameter(default=None, allow_None=True, doc="""
         Optional maximum allowable value.""")
 
-    _rename = {'name': 'title', 'start': 'low', 'end': 'high'}
+    _rename: ClassVar[Mapping[str, str | None]] = {'name': 'title', 'start': 'low', 'end': 'high'}
 
-    _widget_type = _BkNumericInput
+    _widget_type: ClassVar[Type[Model]] = _BkNumericInput
 
     __abstract = True
 
@@ -505,12 +543,12 @@ class _SpinnerBase(_NumericInputBase):
         Defines the debounce time in ms before updating `value_throttled` when
         the mouse wheel is used to change the input.""")
 
-    _widget_type = _BkSpinner
+    _widget_type: ClassVar[Type[Model]] = _BkSpinner
 
     __abstract = True
 
     def __init__(self, **params):
-        if params.get('value') is None:
+        if 'value' not in params:
             value = params.get('start', self.value)
             if value is not None:
                 params['value'] = value
@@ -522,7 +560,10 @@ class _SpinnerBase(_NumericInputBase):
         return '{cls}({params})'.format(cls=type(self).__name__,
                                         params=', '.join(param_reprs(self, ['value_throttled'])))
 
-    def _update_model(self, events, msg, root, model, doc, comm):
+    def _update_model(
+        self, events: Dict[str, param.parameterized.Event], msg: Dict[str, Any],
+        root: Model, model: Model, doc: Document, comm: Optional[Comm]
+    ) -> None:
         if 'value_throttled' in msg:
             del msg['value_throttled']
 
@@ -621,6 +662,9 @@ class LiteralInput(Widget):
     >>> LiteralInput(name='Dictionary', value={'key': [1, 2, 3]}, type=dict)
     """
 
+    placeholder = param.String(default='', doc="""
+      Placeholder for empty input field.""")
+
     serializer = param.ObjectSelector(default='ast', objects=['ast', 'json'], doc="""
        The serialization (and deserialization) method to use. 'ast'
        uses ast.literal_eval and 'json' uses json.loads and json.dumps.
@@ -631,13 +675,19 @@ class LiteralInput(Widget):
 
     value = param.Parameter(default=None)
 
-    _rename = {'name': 'title', 'type': None, 'serializer': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'name': 'title', 'type': None, 'serializer': None
+    }
 
-    _source_transforms = {'value': """JSON.parse(value.replace(/'/g, '"'))"""}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': """JSON.parse(value.replace(/'/g, '"'))"""
+    }
 
-    _target_transforms = {'value': """JSON.stringify(value).replace(/,/g, ",").replace(/:/g, ": ")"""}
+    _target_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': """JSON.stringify(value).replace(/,/g, ",").replace(/:/g, ": ")"""
+    }
 
-    _widget_type = _BkTextInput
+    _widget_type: ClassVar[Type[Model]] = _BkTextInput
 
     def __init__(self, **params):
         super().__init__(**params)
@@ -662,7 +712,9 @@ class LiteralInput(Widget):
         if 'value' in msg:
             value = msg.pop('value')
             try:
-                if self.serializer == 'json':
+                if value == '':
+                    value = ''
+                elif self.serializer == 'json':
                     value = json.loads(value)
                 else:
                     value = ast.literal_eval(value)
@@ -680,7 +732,9 @@ class LiteralInput(Widget):
                             pass
                         else:
                             break
-                    if typed_value is None and value is not None:
+                    if typed_value is None and value == '':
+                        value = None
+                    elif typed_value is None and value is not None:
                         new_state = ' (wrong type)'
                         value = self.value
                     else:
@@ -728,9 +782,9 @@ class ArrayInput(LiteralInput):
         restriction helps avoid overwhelming the browser and lets
         other widgets remain usable.""")
 
-    _rename = dict(LiteralInput._rename, max_array_size=None)
+    _rename: ClassVar[Mapping[str, str | None]] = dict(LiteralInput._rename, max_array_size=None)
 
-    _source_transforms = {'value': None}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {'value': None}
 
     def __init__(self, **params):
         super().__init__(**params)
@@ -801,10 +855,14 @@ class DatetimeInput(LiteralInput):
 
     type = datetime
 
-    _source_transforms = {'value': None, 'start': None, 'end': None}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': None, 'start': None, 'end': None
+    }
 
-    _rename = {'format': None, 'type': None, 'name': 'title',
-               'start': None, 'end': None, 'serializer': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'format': None, 'type': None, 'name': 'title', 'start': None,
+        'end': None, 'serializer': None
+    }
 
     def __init__(self, **params):
         super().__init__(**params)
@@ -885,7 +943,7 @@ class DatetimeRangeInput(CompositeWidget):
     format = param.String(default='%Y-%m-%d %H:%M:%S', doc="""
         Datetime format used for parsing and formatting the datetime.""")
 
-    _composite_type = Column
+    _composite_type: ClassVar[Type[Panel]] = Column
 
     def __init__(self, **params):
         self._text = StaticText(margin=(5, 0, 0, 0), style={'white-space': 'nowrap'})
@@ -958,15 +1016,19 @@ class Checkbox(Widget):
     value = param.Boolean(default=False, doc="""
         The current value""")
 
-    _supports_embed = True
+    _supports_embed: ClassVar[bool] = True
 
-    _rename = {'value': 'active', 'name': 'labels'}
+    _rename: ClassVar[Mapping[str, str | None]] = {'value': 'active', 'name': 'labels'}
 
-    _source_transforms = {'value': "value.indexOf(0) >= 0", 'name': "value[0]"}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': "value.indexOf(0) >= 0", 'name': "value[0]"
+    }
 
-    _target_transforms = {'value': "value ? [0] : []", 'name': "[value]"}
+    _target_transforms: ClassVar[Mapping[str, str | None]] = {
+        'value': "value ? [0] : []", 'name': "[value]"
+    }
 
-    _widget_type = _BkCheckboxGroup
+    _widget_type: ClassVar[Type[Model]] = _BkCheckboxGroup
 
     def _process_property_change(self, msg):
         msg = super()._process_property_change(msg)
